package com.jty.wsxt.infrastructure.security.embed;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.jty.wsxt.infrastructure.code.BodyReaderHttpServletRequestWrapper;
import com.jty.wsxt.infrastructure.security.AuthenticationBean;
import com.jty.wsxt.infrastructure.support.BusinessException;
import com.jty.wsxt.infrastructure.support.Result;
import com.jty.wsxt.infrastructure.support.ResultCode;
import com.jty.wsxt.infrastructure.util.HeadUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.http.MediaType;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.core.Authentication;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.common.exceptions.UnapprovedClientAuthenticationException;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.security.oauth2.provider.OAuth2Request;
import org.springframework.security.oauth2.provider.TokenRequest;
import org.springframework.security.oauth2.provider.client.BaseClientDetails;
import org.springframework.security.oauth2.provider.token.AuthorizationServerTokenServices;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;

/**
 * 嵌入认证过滤器
 * 根据ClientId和ClientSecret验证是否是嵌入
 *
 *
 * @author Jason
 * @since 2019/2/26 13:28
 */
@Component
public class EmbedAuthenticationFilter extends OncePerRequestFilter implements InitializingBean {

    private String usernameParameter = "username";

    @Autowired
    private AuthenticationManager authenticationManager;
    @Autowired
    @Qualifier("defaultAuthorizationServerTokenServices")
    private AuthorizationServerTokenServices authorizationServerTokenServices;

    @Autowired
    private ObjectMapper objectMapper;

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain) throws ServletException, IOException {
        BodyReaderHttpServletRequestWrapper wrapper=new BodyReaderHttpServletRequestWrapper(request);
        if(!"/auth/authentication/embed".equalsIgnoreCase(wrapper.getRequestURI())){
            filterChain.doFilter(wrapper, response);
            return;
        }
        // todo 核查IP访问
        String header = wrapper.getHeader("Authorization");
        // 是否以Basic开头
        if (header == null || !header.startsWith("Basic ")) {
            throw new UnapprovedClientAuthenticationException("请求头中无client信息");
        }
        String[] tokens = HeadUtils.extractAndDecodeHeader(header, wrapper);
        assert tokens.length == 2;
        String clientId = tokens[0];
        String clientSecret = tokens[1];
        String userName;
        if(StringUtils.equalsIgnoreCase(MediaType.APPLICATION_JSON_UTF8_VALUE,request.getContentType())
                ||StringUtils.equalsIgnoreCase(MediaType.APPLICATION_JSON_VALUE,request.getContentType())){
            try (InputStream is = request.getInputStream()){
                ObjectMapper mapper = new ObjectMapper();
                AuthenticationBean authenticationBean = mapper.readValue(is, AuthenticationBean.class);
                userName=authenticationBean.getUsername();
            }
        }else{
            userName=obtainUsername(wrapper);
        }
        if(StringUtils.isEmpty(userName))throw  new BusinessException(ResultCode.PARAM_IS_INVALID);
        EmbedAuthenticationToken token=new EmbedAuthenticationToken(userName);
        token.setSchoolId(1);

        Authentication authentication=authenticationManager.authenticate(token);

        BaseClientDetails clientDetails=new BaseClientDetails();
//        AuthClient client=platform.getClient();
//        if(client==null){
//            throw new BusinessException(ResultCode.CLIENT_NOT_EXIST);
//        }
        clientDetails.setClientId("1");
        //clientDetails.setClientSecret(client.getClientSecret());
        clientDetails.setAuthorizedGrantTypes(Arrays.asList("embed".split(",")));
        clientDetails.setScope(Arrays.asList("all".split(",")));

        TokenRequest tokenRequest = new TokenRequest(MapUtils.EMPTY_MAP,
                clientId,
                clientDetails.getScope(),
                "embed");

        OAuth2Request oAuth2Request = tokenRequest.createOAuth2Request(clientDetails);
        OAuth2Authentication oAuth2Authentication = new OAuth2Authentication(oAuth2Request,authentication);
        OAuth2AccessToken oAuth2AccessToken = authorizationServerTokenServices.createAccessToken(oAuth2Authentication);

        response.setContentType("application/json;charset=UTF-8");
        response.getWriter().write(objectMapper.writeValueAsString(Result.success(oAuth2AccessToken)));


    }



    protected String obtainUsername(HttpServletRequest request) {
        return request.getParameter(this.usernameParameter);
    }
}
