package com.jty.wsxt.interfaces.controller;

import com.jty.wsxt.application.ApplicationRegistry;
import com.jty.wsxt.domain.model.auth.role.AuthRole;
import com.jty.wsxt.infrastructure.support.Result;
import com.jty.wsxt.interfaces.Assembler;
import com.jty.wsxt.interfaces.dto.AuthRoleDto;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;

/**
 * AuthRoleController
 *
 * @author Manjiajie
 * @since 2019-2-13 19:19:46
 */
@RestController
public class AuthRoleController {

    @PostMapping("/role")
    @PreAuthorize("hasAuthority('NRSC_SYSTEM_ROLE')")
    public Result addRole(@RequestBody @Valid AuthRoleDto roleDto){
        ApplicationRegistry.authRoleServer().addRole(roleDto,"nrsc");
      return Result.success();
    }

    @PutMapping("/role/{roleId}")
    @PreAuthorize("hasAuthority('NRSC_SYSTEM_ROLE')")
    public Result updateRole(@PathVariable Integer roleId, @RequestBody @Valid AuthRoleDto roleDto){
        roleDto.setId(roleId);
        ApplicationRegistry.authRoleServer().updateRole(roleDto,"nrsc");
        return Result.success();
    }

    @GetMapping("/role/{roleId}")
    @PreAuthorize("hasAuthority('NRSC_SYSTEM_ROLE')")
    public Result getAuthDetail(@PathVariable Integer roleId){
       AuthRole role = ApplicationRegistry.authRoleServer().getRoleById(roleId);
       return Result.success(Assembler.cashToAuthRoleDto(role));
    }

    @GetMapping("/roles")
    public Result getAuthRoles(AuthRoleDto roleDto){
       Page<AuthRole> pageRole = ApplicationRegistry.authRoleServer().findPageRolesByPlatform("nrsc",roleDto);
        List<AuthRoleDto> roleDtos = new ArrayList<>();
        if(pageRole.getTotalElements() != 0){
            pageRole.getContent().forEach(authRole -> roleDtos.add(Assembler.cashToAuthRoleDto(authRole)));
        }
        return Result.success(new PageImpl<>(roleDtos,pageRole.getPageable(),pageRole.getTotalElements()));
    }
}
