package com.jty.wsxt.infrastructure.util;

import com.aliyun.oss.OSSClient;
import com.aliyun.oss.model.GeneratePresignedUrlRequest;
import com.aliyun.oss.model.ObjectMetadata;
import com.aliyun.oss.model.ResponseHeaderOverrides;
import com.jty.wsxt.infrastructure.support.AlibabaConstants;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.Date;

/**
 * TODO
 *
 * @author Jason
 * @since 2019/1/7 14:50
 */
public class OssUtil {

    public static String uploadMultipartFile(MultipartFile file,String bucket) throws IOException {
        ObjectMetadata meta = new ObjectMetadata();
        meta.setContentType("text/plain");
        meta.setContentDisposition("attachment;filename=\""+file.getOriginalFilename()+"\"");

        OSSClient ossClient = new OSSClient(AlibabaConstants.OSS_endpoint, AlibabaConstants.accessKeyId, AlibabaConstants.accessKeySecret);
        String objectName= LocalDateTime.now().toInstant(ZoneOffset.UTC).toEpochMilli() + RandomStringUtils.randomNumeric(8);

        ossClient.putObject(bucket,objectName,file.getInputStream(), meta);
        ossClient.shutdown();
        return "http://" + bucket + ".oss-cn-beijing.aliyuncs.com/"+objectName;
    }


    public static String getAgingUrl(String bucketName,String key){
        OSSClient ossClient = new OSSClient( AlibabaConstants.OSS_endpoint, AlibabaConstants.accessKeyId, AlibabaConstants.accessKeySecret );

        // 5 minute to expire
        Date expires = new Date( System.currentTimeMillis() + 1000 * 60*60);

        GeneratePresignedUrlRequest generatePresignedUrlRequest =

                new GeneratePresignedUrlRequest(bucketName, key);

        generatePresignedUrlRequest.setExpiration( expires );

       return ossClient.generatePresignedUrl(generatePresignedUrlRequest).toString();
    }

    /**
     * 设置下载文件名
     * @param bucketName
     * @param key
     * @param fileName
     * @return
     */
    public static String getAgingUrl(String bucketName,String key,String fileName){
        OSSClient ossClient = new OSSClient( AlibabaConstants.OSS_endpoint, AlibabaConstants.accessKeyId, AlibabaConstants.accessKeySecret );

        // 5 minute to expire
        Date expires = new Date( System.currentTimeMillis() + 1000 * 60*60);

        GeneratePresignedUrlRequest generatePresignedUrlRequest =

                new GeneratePresignedUrlRequest(bucketName, key);

        generatePresignedUrlRequest.setExpiration( expires );

        //设置现在文件名和文件标题名一致
        if(StringUtils.isNotBlank(fileName)){
            ResponseHeaderOverrides responseHeaders = new ResponseHeaderOverrides();
            responseHeaders.setContentDisposition(" attachment; filename=" + fileName);
            generatePresignedUrlRequest.setResponseHeaders(responseHeaders);
        }


        return ossClient.generatePresignedUrl(generatePresignedUrlRequest).toString();
    }

    public static Boolean checkDocumentExist(String bucketName,String key){
        OSSClient ossClient = new OSSClient(AlibabaConstants.OSS_endpoint, AlibabaConstants.accessKeyId, AlibabaConstants.accessKeySecret);
        // 判断文件是否存在。doesObjectExist还有一个参数isOnlyInOSS，如果为true则忽略302重定向或镜像；如果 为false，则考虑302重定向或镜像。
       return  ossClient.doesObjectExist(bucketName, key);
    }
}
